@@ -76,6 +76,9 @@ static struct viewers {
 	{ NULL, VIEWER_USER_DEFINED },
 };
 
+/* Is the session we are trying to view is in live mode. */
+static int session_live_mode;
+
 /*
  * usage
  */
@@ -88,6 +91,7 @@ static void usage(FILE *ofp)
 	fprintf(ofp, "Where SESSION_NAME is an optional session name. If not specified, lttng will\n");
 	fprintf(ofp, "get it from the configuration file (.lttngrc).\n");
 	fprintf(ofp, "\n");
+	fprintf(ofp, "Options:\n");
 	fprintf(ofp, "  -h, --help               Show this help\n");
 	fprintf(ofp, "      --list-options       Simple listing of options\n");
 	fprintf(ofp, "  -t, --trace-path PATH    Trace directory path for the viewer\n");
@@ -147,7 +151,7 @@ static char **alloc_argv_from_user_opts(char *opts, const char *trace_path)
 	} while (*token != '\0');
 
 	/* Add two here for the NULL terminating element and trace path */
-	argv = malloc(sizeof(char *) * (num_opts + 2));
+	argv = zmalloc(sizeof(char *) * (num_opts + 2));
 	if (argv == NULL) {
 		goto error;
 	}
@@ -155,6 +159,9 @@ static char **alloc_argv_from_user_opts(char *opts, const char *trace_path)
 	token = strtok(opts, " ");
 	while (token != NULL) {
 		argv[i] = strdup(token);
+		if (argv[i] == NULL) {
+			goto error;
+		}
 		token = strtok(NULL, " ");
 		i++;
 	}
@@ -165,6 +172,13 @@ static char **alloc_argv_from_user_opts(char *opts, const char *trace_path)
 	return argv;
 
 error:
+	if (argv) {
+		for (i = 0; i < num_opts + 2; i++) {
+			free(argv[i]);
+		}
+		free(argv);
+	}
+
 	return NULL;
 }
 
@@ -178,20 +192,38 @@ static char **alloc_argv_from_local_opts(const char **opts, size_t opts_len,
 		const char *trace_path)
 {
 	char **argv;
-	size_t size;
+	size_t size, mem_len;
 
-	size = sizeof(char *) * opts_len;
+
+	/* Add one for the NULL terminating element. */
+	mem_len = opts_len + 1;
+	if (session_live_mode) {
+		/* Add 3 option for the live mode being "-i lttng-live URL". */
+		mem_len += 3;
+	} else {
+		/* Add option for the trace path. */
+		mem_len += 1;
+	}
+
+	size = sizeof(char *) * mem_len;
 
 	/* Add two here for the trace_path and the NULL terminating element. */
-	argv = malloc(size + 2);
+	argv = zmalloc(size);
 	if (argv == NULL) {
 		goto error;
 	}
 
 	memcpy(argv, opts, size);
 
-	argv[opts_len] = (char *)trace_path;
-	argv[opts_len + 1] = NULL;
+	if (session_live_mode) {
+		argv[opts_len] = "-i";
+		argv[opts_len + 1] = "lttng-live";
+		argv[opts_len + 2] = (char *) trace_path;
+		argv[opts_len + 3] = NULL;
+	} else {
+		argv[opts_len] = (char *) trace_path;
+		argv[opts_len + 1] = NULL;
+	}
 
 error:
 	return argv;
@@ -272,13 +304,39 @@ error:
 	return ret;
 }
 
+/*
+ * Build the live path we need for the lttng live view.
+ */
+static char *build_live_path(char *session_name)
+{
+	int ret;
+	char *path = NULL;
+	char hostname[HOST_NAME_MAX];
+
+	ret = gethostname(hostname, sizeof(hostname));
+	if (ret < 0) {
+		perror("gethostname");
+		goto error;
+	}
+
+	ret = asprintf(&path, "net://localhost/host/%s/%s", hostname,
+			session_name);
+	if (ret < 0) {
+		perror("asprintf live path");
+		goto error;
+	}
+
+error:
+	return path;
+}
+
 /*
  * Exec viewer if found and use session name path.
  */
 static int view_trace(void)
 {
-	int ret, count, i, found = 0;
-	char *session_name, *trace_path;
+	int ret;
+	char *session_name, *trace_path = NULL;
 	struct lttng_session *sessions = NULL;
 
 	/*
@@ -313,6 +371,8 @@ static int view_trace(void)
 	DBG("Viewing trace for session %s", session_name);
 
 	if (session_name) {
+		int i, count, found = 0;
+
 		/* Getting all sessions */
 		count = lttng_list_sessions(&sessions);
 		if (count < 0) {
@@ -337,7 +397,28 @@ static int view_trace(void)
 			goto free_sessions;
 		}
 
-		trace_path = sessions[i].path;
+		session_live_mode = sessions[i].live_timer_interval;
+
+		DBG("Session live mode set to %d", session_live_mode);
+
+		if (sessions[i].enabled && !session_live_mode) {
+			WARN("Session %s is running. Please stop it before reading it.",
+					session_name);
+			ret = CMD_ERROR;
+			goto free_sessions;
+		}
+
+		/* If the timer interval is set we are in live mode. */
+		if (session_live_mode) {
+			trace_path = build_live_path(session_name);
+			if (!trace_path) {
+				ret = CMD_ERROR;
+				goto free_sessions;
+			}
+		} else {
+			/* Get file system session path. */
+			trace_path = sessions[i].path;
+		}
 	} else {
 		trace_path = opt_trace_path;
 	}
@@ -351,9 +432,10 @@ static int view_trace(void)
 	}
 
 free_sessions:
-	if (sessions) {
-		free(sessions);
+	if (session_live_mode) {
+		free(trace_path);
 	}
+	free(sessions);
 free_error:
 	if (opt_session_name == NULL) {
 		free(session_name);
@@ -373,6 +455,10 @@ int cmd_view(int argc, const char **argv)
 	pc = poptGetContext(NULL, argc, argv, long_options, 0);
 	poptReadDefaultConfig(pc, 0);
 
+	if (lttng_opt_mi) {
+		WARN("mi does not apply to view command");
+	}
+
 	while ((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_HELP:
