@@ -39,7 +39,10 @@ static char *opt_sessiond_path;
 static pid_t sessiond_pid;
 static volatile int recv_child_signal;
 
+char *opt_relayd_path;
+
 enum {
+	OPT_RELAYD_PATH,
 	OPT_SESSION_PATH,
 	OPT_DUMP_OPTIONS,
 	OPT_DUMP_COMMANDS,
@@ -47,12 +50,15 @@ enum {
 
 /* Getopt options. No first level command. */
 static struct option long_options[] = {
+	{"version",          0, NULL, 'V'},
 	{"help",             0, NULL, 'h'},
 	{"group",            1, NULL, 'g'},
 	{"verbose",          0, NULL, 'v'},
 	{"quiet",            0, NULL, 'q'},
+	{"mi",               1, NULL, 'm'},
 	{"no-sessiond",      0, NULL, 'n'},
 	{"sessiond-path",    1, NULL, OPT_SESSION_PATH},
+	{"relayd-path",      1, NULL, OPT_RELAYD_PATH},
 	{"list-options",     0, NULL, OPT_DUMP_OPTIONS},
 	{"list-commands",    0, NULL, OPT_DUMP_COMMANDS},
 	{NULL, 0, NULL, 0}
@@ -74,39 +80,50 @@ static struct cmd_struct commands[] =  {
 	{ "version", cmd_version},
 	{ "calibrate", cmd_calibrate},
 	{ "view", cmd_view},
+	{ "snapshot", cmd_snapshot},
+	{ "save", cmd_save},
+	{ "load", cmd_load},
 	{ NULL, NULL}	/* Array closure */
 };
 
 static void usage(FILE *ofp)
 {
-	fprintf(ofp, "LTTng Trace Control " VERSION" - " VERSION_NAME"\n\n");
-	fprintf(ofp, "usage: lttng [OPTIONS] <COMMAND>\n");
+	fprintf(ofp, "LTTng Trace Control " VERSION " - " VERSION_NAME "%s\n\n",
+		GIT_VERSION[0] == '\0' ? "" : " - " GIT_VERSION);
+	fprintf(ofp, "usage: lttng [OPTIONS] <COMMAND> [<ARGS>]\n");
 	fprintf(ofp, "\n");
 	fprintf(ofp, "Options:\n");
+	fprintf(ofp, "  -V, --version              Show version\n");
 	fprintf(ofp, "  -h, --help                 Show this help\n");
 	fprintf(ofp, "      --list-options         Simple listing of lttng options\n");
 	fprintf(ofp, "      --list-commands        Simple listing of lttng commands\n");
 	fprintf(ofp, "  -v, --verbose              Increase verbosity\n");
 	fprintf(ofp, "  -q, --quiet                Quiet mode\n");
+	fprintf(ofp, "  -m, --mi TYPE              Machine Interface mode.\n");
+	fprintf(ofp, "                                 Type: xml\n");
 	fprintf(ofp, "  -g, --group NAME           Unix tracing group name. (default: tracing)\n");
 	fprintf(ofp, "  -n, --no-sessiond          Don't spawn a session daemon\n");
 	fprintf(ofp, "      --sessiond-path PATH   Session daemon full path\n");
+	fprintf(ofp, "      --relayd-path PATH     Relayd daemon full path\n");
 	fprintf(ofp, "\n");
 	fprintf(ofp, "Commands:\n");
-	fprintf(ofp, "    add-context     Add context to event and/or channel\n");
-	fprintf(ofp, "    calibrate       Quantify LTTng overhead\n");
-	fprintf(ofp, "    create          Create tracing session\n");
-	fprintf(ofp, "    destroy         Tear down tracing session\n");
-	fprintf(ofp, "    enable-channel  Enable tracing channel\n");
-	fprintf(ofp, "    enable-event    Enable tracing event\n");
-	fprintf(ofp, "    disable-channel Disable tracing channel\n");
-	fprintf(ofp, "    disable-event   Disable tracing event\n");
-	fprintf(ofp, "    list            List possible tracing options\n");
-	fprintf(ofp, "    set-session     Set current session name\n");
-	fprintf(ofp, "    start           Start tracing\n");
-	fprintf(ofp, "    stop            Stop tracing\n");
-	fprintf(ofp, "    version         Show version information\n");
-	fprintf(ofp, "    view            Start trace viewer\n");
+	fprintf(ofp, "    add-context       Add context to event and/or channel\n");
+	fprintf(ofp, "    calibrate         Quantify LTTng overhead\n");
+	fprintf(ofp, "    create            Create tracing session\n");
+	fprintf(ofp, "    destroy           Tear down tracing session\n");
+	fprintf(ofp, "    enable-channel    Enable tracing channel\n");
+	fprintf(ofp, "    enable-event      Enable tracing event\n");
+	fprintf(ofp, "    disable-channel   Disable tracing channel\n");
+	fprintf(ofp, "    disable-event     Disable tracing event\n");
+	fprintf(ofp, "    list              List possible tracing options\n");
+	fprintf(ofp, "    set-session       Set current session name\n");
+	fprintf(ofp, "    snapshot          Snapshot buffers of current session name\n");
+	fprintf(ofp, "    start             Start tracing\n");
+	fprintf(ofp, "    stop              Stop tracing\n");
+	fprintf(ofp, "    version           Show version information\n");
+	fprintf(ofp, "    view              Start trace viewer\n");
+	fprintf(ofp, "    save              Save session configuration\n");
+	fprintf(ofp, "    load              Load session configuration\n");
 	fprintf(ofp, "\n");
 	fprintf(ofp, "Each command also has its own -h, --help option.\n");
 	fprintf(ofp, "\n");
@@ -114,6 +131,32 @@ static void usage(FILE *ofp)
 	fprintf(ofp, "See http://lttng.org for updates, bug reports and news.\n");
 }
 
+static void version(FILE *ofp)
+{
+	fprintf(ofp, "%s (LTTng Trace Control) " VERSION" - " VERSION_NAME "%s\n",
+			progname,
+			GIT_VERSION[0] == '\0' ? "" : " - " GIT_VERSION);
+}
+
+/*
+ * Find the MI output type enum from a string. This function is for the support
+ * of machine interface output.
+ */
+static int mi_output_type(const char *output_type)
+{
+	int ret = 0;
+
+	if (!strncasecmp("xml", output_type, 3)) {
+		ret = LTTNG_MI_XML;
+	} else {
+		/* Invalid output format */
+		ERR("MI output format not supported");
+		ret = -LTTNG_ERR_MI_OUTPUT_TYPE;
+	}
+
+	return ret;
+}
+
 /*
  *  list_options
  *
@@ -138,25 +181,6 @@ static void list_options(FILE *ofp)
 	}
 }
 
-/*
- *  list_commands
- *
- *  List commands line by line. This is mostly for bash auto completion and to
- *  avoid difficult parsing.
- */
-static void list_commands(FILE *ofp)
-{
-	int i = 0;
-	struct cmd_struct *cmd = NULL;
-
-	cmd = &commands[i];
-	while (cmd->name != NULL) {
-		fprintf(ofp, "%s\n", cmd->name);
-		i++;
-		cmd = &commands[i];
-	}
-}
-
 /*
  * clean_exit
  */
@@ -270,7 +294,7 @@ static int handle_command(int argc, char **argv)
 	}
 
 	/* Command not found */
-	ret = -1;
+	ret = CMD_UNDEFINED;
 
 end:
 	return ret;
@@ -344,41 +368,44 @@ end:
 static int check_sessiond(void)
 {
 	int ret;
-	char *pathname = NULL, *alloc_pathname = NULL;
+	char *pathname = NULL;
 
 	ret = lttng_session_daemon_alive();
 	if (ret == 0) {	/* not alive */
 		/* Try command line option path */
-		if (opt_sessiond_path != NULL) {
-			ret = access(opt_sessiond_path, F_OK | X_OK);
-			if (ret < 0) {
-				ERR("No such file or access denied: %s", opt_sessiond_path);
-				goto end;
-			}
-			pathname = opt_sessiond_path;
-		} else {
-			/* Try LTTNG_SESSIOND_PATH env variable */
+		pathname = opt_sessiond_path;
+
+		/* Try LTTNG_SESSIOND_PATH env variable */
+		if (pathname == NULL) {
 			pathname = getenv(DEFAULT_SESSIOND_PATH_ENV);
 		}
 
-		/* Let's rock and roll */
+		/* Try with configured path */
 		if (pathname == NULL) {
-			ret = asprintf(&alloc_pathname, INSTALL_BIN_PATH "/lttng-sessiond");
-			if (ret < 0) {
-				perror("asprintf spawn sessiond");
-				goto end;
+			if (CONFIG_SESSIOND_BIN[0] != '\0') {
+				pathname = CONFIG_SESSIOND_BIN;
 			}
-			pathname = alloc_pathname;
+		}
+
+		/* Let's rock and roll while trying the default path */
+		if (pathname == NULL) {
+			pathname = INSTALL_BIN_PATH "/lttng-sessiond";
+		}
+
+		DBG("Session daemon at: %s", pathname);
+
+		/* Check existence and permissions */
+		ret = access(pathname, F_OK | X_OK);
+		if (ret < 0) {
+			ERR("No such file or access denied: %s", pathname);
+			goto end;
 		}
 
 		ret = spawn_sessiond(pathname);
-		free(alloc_pathname);
 		if (ret < 0) {
 			ERR("Problem occurred when starting %s", pathname);
-			goto end;
 		}
 	}
-
 end:
 	return ret;
 }
@@ -415,24 +442,39 @@ static int check_args_no_sessiond(int argc, char **argv)
 static int parse_args(int argc, char **argv)
 {
 	int opt, ret;
+	char *user;
 
 	if (argc < 2) {
 		usage(stderr);
 		clean_exit(EXIT_FAILURE);
 	}
 
-	while ((opt = getopt_long(argc, argv, "+hnvqg:", long_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "+Vhnvqg:m:", long_options, NULL)) != -1) {
 		switch (opt) {
+		case 'V':
+			version(stdout);
+			ret = 0;
+			goto end;
 		case 'h':
 			usage(stdout);
 			ret = 0;
 			goto end;
 		case 'v':
-			lttng_opt_verbose += 1;
+			/* There is only 3 possible level of verbosity. (-vvv) */
+			if (lttng_opt_verbose < 3) {
+				lttng_opt_verbose += 1;
+			}
 			break;
 		case 'q':
 			lttng_opt_quiet = 1;
 			break;
+		case 'm':
+			lttng_opt_mi = mi_output_type(optarg);
+			if (lttng_opt_mi < 0) {
+				ret = lttng_opt_mi;
+				goto error;
+			}
+			break;
 		case 'g':
 			lttng_set_tracing_group(optarg);
 			break;
@@ -441,13 +483,24 @@ static int parse_args(int argc, char **argv)
 			break;
 		case OPT_SESSION_PATH:
 			opt_sessiond_path = strdup(optarg);
+			if (!opt_sessiond_path) {
+				ret = -1;
+				goto error;
+			}
+			break;
+		case OPT_RELAYD_PATH:
+			opt_relayd_path = strdup(optarg);
+			if (!opt_relayd_path) {
+				ret = -1;
+				goto error;
+			}
 			break;
 		case OPT_DUMP_OPTIONS:
 			list_options(stdout);
 			ret = 0;
 			goto end;
 		case OPT_DUMP_COMMANDS:
-			list_commands(stdout);
+			list_commands(commands, stdout);
 			ret = 0;
 			goto end;
 		default:
@@ -476,6 +529,14 @@ static int parse_args(int argc, char **argv)
 		goto error;
 	}
 
+	/* For Mathieu Desnoyers a.k.a. Dr. Tracing */
+	user = getenv("USER");
+	if (user != NULL && ((strncmp(progname, "drtrace", 7) == 0 ||
+					strncmp("compudj", user, 7) == 0))) {
+		MSG("%c[%d;%dmWelcome back Dr Tracing!%c[%dm\n", 27,1,33,27,0);
+	}
+	/* Thanks Mathieu */
+
 	/* 
 	 * Handle leftovers which is a first level command with the trailing
 	 * options.
@@ -494,6 +555,9 @@ static int parse_args(int argc, char **argv)
 	case CMD_FATAL:
 		ERR("Fatal error");
 		break;
+	case CMD_UNSUPPORTED:
+		ERR("Unsupported command");
+		break;
 	case -1:
 		usage(stderr);
 		ret = 1;
@@ -519,18 +583,9 @@ error:
 int main(int argc, char *argv[])
 {
 	int ret;
-	char *user;
 
 	progname = argv[0] ? argv[0] : "lttng";
 
-	/* For Mathieu Desnoyers a.k.a. Dr. Tracing */
-	user = getenv("USER");
-	if (user != NULL && ((strncmp(progname, "drtrace", 7) == 0 ||
-				strncmp("compudj", user, 7) == 0))) {
-		MSG("%c[%d;%dmWelcome back Dr Tracing!%c[%dm\n", 27,1,33,27,0);
-	}
-	/* Thanks Mathieu */
-
 	ret = set_signal_handler();
 	if (ret < 0) {
 		clean_exit(ret);
