@@ -20,19 +20,38 @@
 
 #include <errno.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <string.h>
+#include <urcu/tls-compat.h>
+#include <time.h>
 
 #ifndef _GNU_SOURCE
 #error "lttng-tools error.h needs _GNU_SOURCE"
 #endif
 
+#include <lttng/lttng-error.h>
+#include <common/compat/tid.h>
+
 /* Stringify the expansion of a define */
 #define XSTR(d) STR(d)
 #define STR(s) #s
 
+/*
+ * Contains the string of the log entry time. This is used as a thread local
+ * storage so we don't race between thread and also avoid memory allocation
+ * every time a log is fired.
+ */
+struct log_time {
+	/* Format: 00:00:00.000000 plus NULL byte. */
+	char str[16];
+};
+extern DECLARE_URCU_TLS(struct log_time, error_log_time);
+
 extern int lttng_opt_quiet;
 extern int lttng_opt_verbose;
+extern int lttng_opt_mi;
 
+/* Error type. */
 #define PRINT_ERR   0x1
 #define PRINT_WARN  0x2
 #define PRINT_BUG   0x3
@@ -43,12 +62,18 @@ extern int lttng_opt_verbose;
 
 /*
  * Macro for printing message depending on command line option and verbosity.
+ *
+ * Machine interface:
+ * We use lttng_opt_mi to suppress all normal msg to stdout. We don't
+ * want any nested msg to show up when printing mi to stdout(if it's the case).
+ * All warnings and errors should be printed to stderr as normal.
  */
 #define __lttng_print(type, fmt, args...)                           \
 	do {                                                            \
-		if (lttng_opt_quiet == 0 && type == PRINT_MSG) {            \
+		if (lttng_opt_quiet == 0 && lttng_opt_mi == 0 &&            \
+				type == PRINT_MSG) {                                \
 			fprintf(stdout, fmt, ## args);                          \
-		} else if (lttng_opt_quiet == 0 &&                          \
+		} else if (lttng_opt_quiet == 0 && lttng_opt_mi == 0 &&     \
 				(((type & PRINT_DBG) && lttng_opt_verbose == 1) ||  \
 				((type & (PRINT_DBG | PRINT_DBG2)) &&               \
 					lttng_opt_verbose == 2) ||                      \
@@ -62,26 +87,27 @@ extern int lttng_opt_verbose;
 		}                                                           \
 	} while (0);
 
+/* Three level of debug. Use -v, -vv or -vvv for the levels */
+#define _ERRMSG(msg, type, fmt, args...) __lttng_print(type, msg \
+		" - %s [%ld/%ld]: " fmt " (in %s() at " __FILE__ ":" XSTR(__LINE__) ")\n", \
+			log_add_time(), (long) getpid(), (long) gettid(), ## args, __func__)
+
 #define MSG(fmt, args...) \
 	__lttng_print(PRINT_MSG, fmt "\n", ## args)
+#define _MSG(fmt, args...) \
+	__lttng_print(PRINT_MSG, fmt, ## args)
 #define ERR(fmt, args...) \
 	__lttng_print(PRINT_ERR, "Error: " fmt "\n", ## args)
 #define WARN(fmt, args...) \
-	__lttng_print(PRINT_WARN, "Warning: " fmt "\n", ## args)
-#define BUG(fmt, args...) \
-	__lttng_print(PRINT_BUG, "BUG: " fmt "\n", ## args)
+	__lttng_print(PRINT_ERR, "Warning: " fmt "\n", ## args)
 
-/* Three level of debug. Use -v, -vv or -vvv for the levels */
-#define DBG(fmt, args...) __lttng_print(PRINT_DBG, "DEBUG1: " fmt \
-		" [in %s() at " __FILE__ ":" XSTR(__LINE__) "]\n", ## args, __func__)
-#define DBG2(fmt, args...) __lttng_print(PRINT_DBG2, "DEBUG2: " fmt \
-		" [in %s() at " __FILE__ ":" XSTR(__LINE__) "]\n", ## args, __func__)
-#define DBG3(fmt, args...) __lttng_print(PRINT_DBG3, "DEBUG3: " fmt \
-		" [in %s() at " __FILE__ ":" XSTR(__LINE__) "]\n", ## args, __func__)
+#define BUG(fmt, args...) _ERRMSG("BUG", PRINT_BUG, fmt, ## args)
 
-#define _PERROR(fmt, args...) \
-	__lttng_print(PRINT_ERR, "PERROR: " fmt \
-		" [in %s() at " __FILE__ ":" XSTR(__LINE__) "]\n", ## args, __func__)
+#define DBG(fmt, args...) _ERRMSG("DEBUG1", PRINT_DBG, fmt, ## args)
+#define DBG2(fmt, args...) _ERRMSG("DEBUG2", PRINT_DBG2, fmt, ## args)
+#define DBG3(fmt, args...) _ERRMSG("DEBUG3", PRINT_DBG3, fmt, ## args)
+
+#define _PERROR(fmt, args...) _ERRMSG("PERROR", PRINT_ERR, fmt, ## args)
 
 #if !defined(__linux__) || ((_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && !defined(_GNU_SOURCE))
 
@@ -107,4 +133,13 @@ extern int lttng_opt_verbose;
 	} while(0);
 #endif
 
+const char *error_get_str(int32_t code);
+
+/*
+ * Function that format the time and return the reference of log_time.str to
+ * the caller. On error, an empty string is returned thus no time will be
+ * printed in the log.
+ */
+const char *log_add_time();
+
 #endif /* _ERROR_H */
